@@ -2,14 +2,14 @@ use ross_dsl::Parser;
 
 fn main() {
     let text = "
-        let time = 0;
+        let time = 0~u32;
 
         const device_address = 0x0002~u16;
 
         do {
             match tick;
             match producer device_address;
-            match { U32IncrementStateFilter(time); }
+            match { IncrementStateByConstFilter(time, 1~u32); }
         }
     ";
 
